@@ -16,11 +16,16 @@
 # specific language governing permissions and limitations
 # under the License.
 """Get code APIs."""
+from __future__ import annotations
+
+from deprecated import deprecated
+
 from airflow.api.common.experimental import check_and_get_dag
 from airflow.exceptions import AirflowException, DagCodeNotFound
 from airflow.models.dagcode import DagCode
 
 
+@deprecated(reason="Use DagCode().get_code_by_fileloc() instead", version="2.2.4")
 def get_code(dag_id: str) -> str:
     """Return python code of a given dag_id.
 
