@@ -15,15 +15,15 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Example DAG demonstrating the usage of the BranchPythonOperator."""
+from __future__ import annotations
 
 import random
 
 import pendulum
 
 from airflow import DAG
-from airflow.operators.dummy import DummyOperator
+from airflow.operators.empty import EmptyOperator
 from airflow.operators.python import BranchPythonOperator
 from airflow.utils.edgemodifier import Label
 from airflow.utils.trigger_rule import TriggerRule
@@ -32,10 +32,10 @@ with DAG(
     dag_id='example_branch_operator',
     start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
     catchup=False,
-    schedule_interval="@daily",
+    schedule="@daily",
     tags=['example', 'example2'],
 ) as dag:
-    run_this_first = DummyOperator(
+    run_this_first = EmptyOperator(
         task_id='run_this_first',
     )
 
@@ -47,19 +47,19 @@ with DAG(
     )
     run_this_first >> branching
 
-    join = DummyOperator(
+    join = EmptyOperator(
         task_id='join',
         trigger_rule=TriggerRule.NONE_FAILED_MIN_ONE_SUCCESS,
     )
 
     for option in options:
-        t = DummyOperator(
+        t = EmptyOperator(
             task_id=option,
         )
 
-        dummy_follow = DummyOperator(
+        empty_follow = EmptyOperator(
             task_id='follow_' + option,
         )
 
         # Label is optional here, but it can help identify more complex branches
-        branching >> Label(option) >> t >> dummy_follow >> join
+        branching >> Label(option) >> t >> empty_follow >> join
