@@ -15,16 +15,40 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import abc
+import enum
 import logging
 import re
 import sys
+from io import IOBase
 from logging import Handler, Logger, StreamHandler
+from typing import IO, cast
 
 # 7-bit C1 ANSI escape sequences
 ANSI_ESCAPE = re.compile(r'\x1B[@-_][0-?]*[ -/]*[@-~]')
 
 
+# Private: A sentinel objects
+class SetContextPropagate(enum.Enum):
+    """:meta private:"""
+
+    # If a `set_context` function wants to _keep_ propagation set on it's logger it needs to return this
+    # special value.
+    MAINTAIN_PROPAGATE = object()
+    # Don't use this one anymore!
+    DISABLE_PROPAGATE = object()
+
+
+def __getattr__(name):
+    if name in ("DISABLE_PROPOGATE", "DISABLE_PROPAGATE"):
+        # Compat for spelling on off chance someone is using this directly
+        # And old object that isn't needed anymore
+        return SetContextPropagate.DISABLE_PROPAGATE
+    raise AttributeError(f"module {__name__} has no attribute {name}")
+
+
 def remove_escape_codes(text: str) -> str:
     """
     Remove ANSI escapes codes from string. It's used to remove
@@ -36,18 +60,17 @@ def remove_escape_codes(text: str) -> str:
 class LoggingMixin:
     """Convenience super-class to have a logger configured with the class name"""
 
+    _log: logging.Logger | None = None
+
     def __init__(self, context=None):
         self._set_context(context)
 
     @property
     def log(self) -> Logger:
         """Returns a logger."""
-        try:
-            # FIXME: LoggingMixin should have a default _log field.
-            return self._log  # type: ignore
-        except AttributeError:
+        if self._log is None:
             self._log = logging.getLogger(self.__class__.__module__ + '.' + self.__class__.__name__)
-            return self._log
+        return self._log
 
     def _set_context(self, context):
         if context is not None:
@@ -72,8 +95,13 @@ class ExternalLoggingMixin:
         """Return whether handler is able to support external links."""
 
 
-# TODO: Formally inherit from io.IOBase
-class StreamLogWriter:
+# We have to ignore typing errors here because Python I/O classes are a mess, and they do not
+# have the same type hierarchy defined as the `typing.IO` - they violate Liskov Substitution Principle
+# While it is ok to make your class derive from IOBase (and its good thing to do as they provide
+# base implementation for IO-implementing classes, it's impossible to make them work with
+# IO generics (and apparently it has not even been intended)
+# See more: https://giters.com/python/typeshed/issues/6077
+class StreamLogWriter(IOBase, IO[str]):  # type: ignore[misc]
     """Allows to redirect stdout and stderr to logger"""
 
     encoding: None = None
@@ -117,15 +145,15 @@ class StreamLogWriter:
         if not message.endswith("\n"):
             self._buffer += message
         else:
-            self._buffer += message
-            self._propagate_log(self._buffer.rstrip())
-            self._buffer = ''
+            self._buffer += message.rstrip()
+            self.flush()
 
     def flush(self):
         """Ensure all logging output has been flushed"""
-        if len(self._buffer) > 0:
-            self._propagate_log(self._buffer)
+        buf = self._buffer
+        if len(buf) > 0:
             self._buffer = ''
+            self._propagate_log(buf)
 
     def isatty(self):
         """
@@ -171,16 +199,24 @@ def set_context(logger, value):
     :param logger: logger
     :param value: value to set
     """
-    _logger = logger
-    while _logger:
-        for handler in _logger.handlers:
-            try:
-                handler.set_context(value)
-            except AttributeError:
-                # Not all handlers need to have context passed in so we ignore
-                # the error when handlers do not have set_context defined.
-                pass
-        if _logger.propagate is True:
-            _logger = _logger.parent
+    while logger:
+        orig_propagate = logger.propagate
+        for handler in logger.handlers:
+            # Not all handlers need to have context passed in so we ignore
+            # the error when handlers do not have set_context defined.
+
+            # Don't use getatrr so we have type checking. And we don't care if handler is actually a
+            # FileTaskHandler, it just needs to have a set_context function!
+            if hasattr(handler, "set_context"):
+                from airflow.utils.log.file_task_handler import FileTaskHandler
+
+                flag = cast(FileTaskHandler, handler).set_context(value)
+                # By default we disable propagate once we have configured the logger, unless that handler
+                # explicitly asks us to keep it on.
+                if flag is not SetContextPropagate.MAINTAIN_PROPAGATE:
+                    logger.propagate = False
+        if orig_propagate is True:
+            # If we were set to propagate before we turned if off, then keep passing set_context up
+            logger = logger.parent
         else:
-            _logger = None
+            break
