@@ -15,6 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 """Common utility functions with strings"""
+from __future__ import annotations
 
 import string
 from random import choice
@@ -25,6 +26,13 @@ def get_random_string(length=8, choices=string.ascii_letters + string.digits):
     return ''.join(choice(choices) for _ in range(length))
 
 
-def to_boolean(astring):
-    """Convert a string to a boolean"""
-    return False if astring is None else astring.lower() in ['true', 't', 'y', 'yes', '1']
+TRUE_LIKE_VALUES = {"on", "t", "true", "y", "yes", "1"}
+
+
+def to_boolean(astring: str | None) -> bool:
+    """Convert a string to a boolean."""
+    if astring is None:
+        return False
+    if astring.lower() in TRUE_LIKE_VALUES:
+        return True
+    return False
